import unittest
from dzy import utils
from freezegun import freeze_time


class TestDW(unittest.TestCase):

    @freeze_time("2021-08-01")
    def test_get_today_str(self):

        self.assertEqual(utils.get_today_str(), "20210801")


if __name__ == '__main__':
    unittest.main()



import logging
from unittest.mock import MagicMock, patch

import pytest

from dzy.keepalive import with_keepalives

asap = 0


class TestWithKeepalives:
    @staticmethod
    @pytest.fixture
    def timer_factory():
        with patch("dzy.keepalive.RecurringTimer") as m:
            yield m

    def test_creates_recurring_timer_with_proper_interval(
        self,
        timer_factory: MagicMock,
    ):
        interval = 123

        @with_keepalives(interval=interval)
        def function():
            pass

        timer_factory.assert_called_once()
        assert ("interval", interval) in (
            timer_factory.call_args_list[0].kwargs.items()
        )

    def test_creates_recurring_timer_that_logs(
        self,
        caplog,
        timer_factory: MagicMock,
    ):
        @with_keepalives()
        def function():
            pass

        timer_factory.assert_called_once()
        timer_callback = timer_factory.call_args_list[0].kwargs["function"]
        with caplog.at_level(logging.INFO):
            timer_callback("anything")

        assert caplog.messages != []

    @pytest.mark.parametrize("funcname", ["start", "cancel", "join"])
    def test_starts_stops_timer_only_when_function_is_called(
        self,
        timer_factory: MagicMock,
        funcname: str
    ):
        @with_keepalives()
        def function():
            pass

        mockedfunc = getattr(timer_factory(), funcname)
        mockedfunc.assert_not_called()

        function()

        mockedfunc.assert_called_once()

    def test_cancels_timer_when_function_raises(
        self,
        timer_factory: MagicMock,
    ):
        @with_keepalives()
        def function():
            raise Exception("test")

        with pytest.raises(Exception):
            function()

        timer_factory().join.assert_called_once()
        timer_factory().cancel.assert_called_once()

    def test_decorated_function_returns_value(self):
        rv = 123

        @with_keepalives()
        def function():
            return rv

        assert function() == rv



import logging
from functools import wraps

from dzy.timer import RecurringTimer

log = logging.getLogger(__name__)


def with_keepalives(*, interval: float = 10):
    timer = RecurringTimer(
        interval=interval,
        function=log.info,
        args=("still alive", ),
    )

    def factory(fn):
        @wraps(fn)
        def wrapper(*a, **kw):
            try:
                timer.start()
                return fn(*a, **kw)
            finally:
                timer.cancel()
                timer.join()
        return wrapper
    return factory



import asyncio
import threading
from functools import partial
from typing import Callable, Dict, Iterable, Optional


class RecurringTimer(threading.Thread):
    def __init__(
        self,
        interval: float,
        function: Callable,
        args: Optional[Iterable] = None,
        kwargs: Optional[Dict] = None,
    ):
        args = args or ()
        kwargs = kwargs or {}
        self._function = partial(function, *args, **kwargs)

        self._interval = interval
        self._evloop = asyncio.new_event_loop()

        super().__init__(None, self._evloop.run_forever)

    def start(self):
        super().start()
        self._evloop.call_soon_threadsafe(self._schedule)

    def cancel(self):
        self._evloop.stop()

    def join(self):
        super().join()
        self._evloop.close()

    def _schedule(self):
        self._evloop.call_later(self._interval, self._tick)

    def _tick(self):
        self._function()
        self._schedule()



import gcsfs
from datetime import datetime
from yaml import safe_load


def get_today_str(date_format: str = "%Y%m%d"):
    """
    Returns today's date as string (based on UTC time).
    """
    return datetime.utcnow().strftime(date_format)


def get_unix_time():
    """
    Returns the current UTC unix time.
    """
    return datetime.utcnow().timestamp()


def read_yaml_file(
        file_path: str,
        fs: gcsfs.GCSFileSystem = None,
        bucket_name: str = None
) -> dict:
    """
    Reads in yaml file, either from locally or from GCS.

    Looks for gs://{bucket_name}/{file_path} when bucket and fs are both
    present, else tries the file_path locally.
    """
    if fs and bucket_name:
        gcs_json_path = f"{bucket_name}/{file_path}"
        with fs.open(gcs_json_path) as f:
            y = safe_load(f)
    else:
        with open(file_path, "r") as f:
            y = safe_load(f)

    f.close()

    return y



from typing import Callable
from unittest.mock import MagicMock

from dzy.timer import RecurringTimer

asap = 0
never = 24 * 3600  # tomorrow (24h) in seconds


def callchain(*chain) -> Callable:
    chain = iter(chain)

    def wrapper():
        fn = next(chain)
        fn = (lambda: None) if isinstance(fn, type(...)) else fn
        return fn()

    return wrapper


class TestRecurringTimer:
    def setup_method(self):
        self.function = MagicMock()

    def test_timer_can_be_canceled_before_first_execution(self):
        timer = RecurringTimer(never, self.function)
        timer.start()

        timer.cancel()
        timer.join()

        self.function.assert_not_called()

    def test_timer_stops_after_few_executions(self):
        timer = RecurringTimer(asap, self.function)
        self.function.side_effect = callchain(..., ..., timer.cancel)

        timer.start()
        timer.join()

        assert self.function.call_count == 3

    def test_timer_passes_arguments(self):
        args = ("1", 2, 3.14)
        kwargs = {"foo": "bar"}
        timer = RecurringTimer(asap, self.function, args, kwargs)
        self.function.side_effect = lambda *a, **kw: timer.cancel()

        timer.start()
        timer.join()

        self.function.assert_called_once_with(*args, **kwargs)
